import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Our main characters -> ANTS
 * 
 * @author  AntMind - Team2
 * @version 13.5.2019
 */
public class Ant extends Animal
{
    private Anthill home;
    private boolean hasFood = false;
    private boolean smellsPheromone = false;
    private static int SMELL_DISTANCE = 25;
    private int smellDistance = SMELL_DISTANCE;
    private int stepSize = 2;
    private int starvation = 5500;
    private int releasePher = 3;
    private static int PREDATOR_DISTANCE = 40;
    private int predatorDistance = PREDATOR_DISTANCE;
    private int life = 20000;
    private boolean checkedHome;
    /**
     * Constructor for creating an ant with an assigned anthill and random orientation
     */
    public Ant(Anthill anthill)
    {
        home = anthill;
        setRotation(Greenfoot.getRandomNumber(360));
    }

    /**
     * Ants try to survive (Escaping from danger, eating.)
     * Then they look for food using pheromone tracks.
     */
    public void act() 
    {

        if((getOneIntersectingObject(Predator.class)!=null)||(getOneIntersectingObject(Player.class)!=null)){
            getWorld().removeObject(this);
            home.getAnts(-1);
            //Too hungry to keep on living.
        }else{
            danger();
            //Is a predator nearby?
            deathToStarvation();
            if(starvation<800 && !checkedHome()){
                goTowards(home);
                //Has to eat something.
                if(atHome()&&home.getCounterF(0)>0){
                    starvation += 4000+Greenfoot.getRandomNumber(1000);
                    home.getCounterF(-1);
                }
            }
            else{
                if(hasFood){ //Am I carrying a food => go home and realease pheromones.
                    checkHome();
                    goTowards(home);
                    releasePheromone();
                }else{
                    Actor food = smellFood(); //Is some food nearby?
                    if(getOneObjectAtOffset(0,0,Pheromone.class)!=null){ 
                        //Whenever I find a pheromones, let's follow them.
                        turnTowards(home.getX(), home.getY());
                        turn(180);
                    }
                    if(food!=null){
                        goTowards(food);
                        //I found food!! Let's take it!!
                        Actor closeFood;
                        closeFood = getOneObjectAtOffset(0,0,HoneyDrop.class);
                        if(closeFood!=null){
                            pickUpFood((HoneyDrop)closeFood);
                        }
                    }else{
                        //JUst moving around
                        if(atWorldEdge()){ // Turning at the edge of the world
                            turn(180);
                        }
                        else if(!goDirect()){    //random walk
                            turn(10-Greenfoot.getRandomNumber(20)); //Turn randomly between -10 and 10 
                        }
                        move(stepSize);
                    }
                }
                starvation--; //I get hungry as well.
            }

            deathToAge(); //My lifespan is not unlimited.
        }
    }

    public boolean goDirect()
    {
        if(getOneObjectAtOffset(0,0,Pheromone.class)!=null){
            return true;
        }else{
            return false;
        }
    }

    /**
     * Makes ant go in a direction.
     */
    public void goTowards(Actor target)
    {
        int x = target.getX();
        int y = target.getY();
        if (getX()==x&&getY()==y){
            //rest, you are there
        }else{
            turnTowards(x, y);
            move(stepSize);
        }
    }

    public void checkHome(){
        if(atHome()) {
            dropFood();
            smellDistance = SMELL_DISTANCE;
            checkedHome=false;
        }
    }

    private boolean atHome()
    {
        if(Math.abs(getX() - home.getX()) < 2*stepSize && 
        Math.abs(getY() - home.getY()) < 2*stepSize){
            return true;
        }
        else{
            return false;
        }
    }

    public void pickUpFood(HoneyDrop food)
    {
        hasFood = true;
        food.takeFood();
        setImage("ant-with-food.png");
    }

    public void dropFood(){
        hasFood=false;
        setImage("ant.png.");
        home.getCounterF(1);
        releasePher=3;
    }

    public void releasePheromone()
    {
        if(releasePher == 3){
            getWorld().addObject(new Pheromone(), getX(), getY());
            releasePher = 0;
        }
        releasePher++;
    }

    public Actor smellFood(){//Smelling HoneyDrops.
        Actor closest = null;
        if(!getObjectsInRange(smellDistance, HoneyDrop.class).isEmpty())
        {
            for(Object obj: getObjectsInRange(smellDistance, HoneyDrop.class))
            {
                Actor target = (Actor) obj;
                int targetDist = (int) Math.sqrt((Math.pow(target.getX()-getX(),2)+Math.pow(target.getY()-getY(),2)));
                if (closest == null || targetDist < smellDistance){
                    closest = target;
                    smellDistance = targetDist;
                }
            }
        }
        return closest;
    }

    public Actor smellPredator(){//Smelling Predator.
        Actor closest = null;
        predatorDistance = PREDATOR_DISTANCE;
        if(!getObjectsInRange(predatorDistance, Predator.class).isEmpty())
        {
            for(Object obj: getObjectsInRange(predatorDistance, (Predator.class)))
            {
                Actor target = (Actor) obj;
                int targetDist = (int) Math.sqrt((Math.pow(target.getX()-getX(),2)+Math.pow(target.getY()-getY(),2)));
                if (closest == null || targetDist < predatorDistance){
                    closest = target;
                    predatorDistance = targetDist;
                }
            }
        }
        return closest;
    }

    public Actor smellPlayer(){ //Smelling Player.
        Actor closest = null;
        predatorDistance = PREDATOR_DISTANCE;
        if(!getObjectsInRange(predatorDistance, Player.class).isEmpty())
        {
            for(Object obj: getObjectsInRange(predatorDistance, (Player.class)))
            {
                Actor target = (Actor) obj;
                int targetDist = (int) Math.sqrt((Math.pow(target.getX()-getX(),2)+Math.pow(target.getY()-getY(),2)));
                if (closest == null || targetDist < predatorDistance){
                    closest = target;
                    predatorDistance = targetDist;
                }
            }
        }
        return closest;
    }

    private void deathToAge() //Dying after spending much time in the arena to its age.
    {
        life--;
        if(life<0)
        {
            Actor skull = new Skull();
            home.getAnts(-1);
            getWorld().addObject(skull, getX(), getY());
            getWorld().removeObject(this);
        }

    }

    private boolean checkedHome()
    {
        checkedHome = false;
        if(atHome()){checkedHome = true;}
        return checkedHome;
    }

    private void danger() //Trying to escape from Predator or PLayer.
    {
        Actor predator = smellPredator();
        if(predator!=null){
            turnTowards(predator.getX(), predator.getY());
            turn(180);
            move(stepSize+Greenfoot.getRandomNumber(5)+2);
            if(atWorldEdge()){turn(60);}

        }
        Actor player = smellPlayer();
        if(player!=null){
            turnTowards(player.getX(), player.getY());
            turn(180);
            move(stepSize+Greenfoot.getRandomNumber(5)+2);
            if(atWorldEdge()){turn(60);}

        }
    }

    private void deathToStarvation() //Dies when its Huneger is too big.
    {
        if(starvation<1){
            if(home.getCounterF(0)<1&&checkedHome()){
                Actor skull = new Skull();
                home.getAnts(-1);
                getWorld().addObject(skull, getX(), getY());
                getWorld().removeObject(this);
            }
        }
    }

}
